//****************************** capture1.c ********************************
//* This is a sample application program that shows you how to use the
//* API function calls of the loadable Linux device driver '615driver'
//* (for Sensoray Model 615) to develop image-grabbing application(s).
//*
//* capture1:	capture from 1 video source
//*
//* Copyright (c) 2003-2004.  Sensoray Co., Inc
//* Author:                   Charlie X. Liu
//*
//*--------------------------------------------------------------------------
//* This program is free software; you can redistribute it and/or
//* modify it under the terms of the GNU General Public License
//* as published by the Free Software Foundation; either version 2
//* of the License, or (at your option) any later version.
//*
//* This program is distributed in the hope that it will be useful,
//* but WITHOUT ANY WARRANTY; without even the implied warranty of
//* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//* GNU General Public License for more details.
//*
//* You should have received a copy of the GNU General Public License
//* along with this program; if not, write to the Free Software
//* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
//* *************************************************************************/


#include <sys/timeb.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <string.h>
#include <sys/mman.h>
#include "615.h"

extern int SvHeaderBMP (int fh, int width, int height, BOOL color);

int main()
{
  int h_File1, h_File2;
  int i, num;
  int width, height;
  char str[200];
  imgstatus status;
  userstatus ustatus;
  BUFFER *pb0, *pb1, *pb2, *pb3;

  int bank1 = open("/dev/CameraBank1", O_RDWR );
  if(bank1 == -1)
  {
    perror("bank1");
    return errno;
  }
  pb0 = (BUFFER*)mmap(0, sizeof(BUFFER), PROT_READ | PROT_WRITE, MAP_SHARED ,bank1 , 0);
  pb1 = (BUFFER*)mmap(0, sizeof(BUFFER), PROT_READ | PROT_WRITE, MAP_SHARED ,bank1 , 0);
  pb2 = (BUFFER*)mmap(0, sizeof(BUFFER), PROT_READ | PROT_WRITE, MAP_SHARED ,bank1 , 0);
  pb3 = (BUFFER*)mmap(0, sizeof(BUFFER), PROT_READ | PROT_WRITE, MAP_SHARED ,bank1 , 0);

  ustatus.jpegon = JPEG_ON;
  ustatus.captionmode = CAP_ON | CAP_MGT;
  ustatus.interlacemode = INT_INT;	//interlaced mode
  memset(ustatus.SEQUENCE, 0, sizeof(ustatus.SEQUENCE));


  // suggested correct sequence sample:   for 1 input/camera		been tested
  //
  // 	SEQUENCE[Input_port_#-1][0] = chan # (0-3)
  // 	SEQUENCE[Input_port_#-1][1] = Assigned camera # (1-16)
  //
  ustatus.SEQUENCE[0][0] = 0;       //  chan   camera   in-port_#
  ustatus.SEQUENCE[0][1] = 1;       //   0  -->  1          1
  ustatus.SEQUENCE[1][0] = 0;
  ustatus.SEQUENCE[1][1] = 0;       //   0  -->  0      terminator


  ustatus.mode.format = FORMAT_NTSC;
  ustatus.mode.bmpflip = FLIP_ON;
  ustatus.mode.bmpcolor = COLOR_RGB;     //   COLOR_RGB
  ustatus.mode.bmpsize = BMP_SIZE_LRG;   // ~ 256 x 192
  ustatus.mode.jpgsize = JPG_SIZE_FULL;  // ~ 704 x 480
  ustatus.mode.captionformat = CAP16X4;
  ustatus.mode.captionpos = CP_LR;
  ustatus.mode.fontsize = CF_SMALL;
  ustatus.mode.compfactor = 0xff;
  ustatus.mode.xpinit = XPS_INIT;

  write(bank1, &ustatus, sizeof(userstatus));

  // for cleaning-up old .jpg and .bmp files.
  system("rm *.jpg");
  system("rm *.bmp");

  ioctl(bank1, S615Start);
  for (i=0; i < 100; i++)
  {
      num = read(bank1, &status, sizeof(imgstatus));
      if(num == sizeof(status))
      {
	  // do stuff with the buffer
	  printf("Reading from buffer #%i i=%i num=%i videopresent = %i camera #%i size= %i\n", status.buffernumber, i, num, status.videopresent, status.camera, status.jpegsize);

	  if(status.videopresent==1)
	  {
	  	//printf("\t\t %i.jpg <== camera %i \n", status.camera, status.camera);
	  	sprintf(str, "%i.jpg", i); //status.camera);
	  	//  sprintf(str, "/home/dstroupe/pictures/%i.jpg", i);
	  	h_File1 = creat (str, S_IRWXU);
	  	write (h_File1, status.pjpg, status.jpegsize);
	  	close (h_File1);

	  	sprintf(str, "%i.bmp", i); //status.camera);
	  	//  sprintf(str, "/home/dstroupe/pictures/%i.bmp", i);
	 	 h_File2 = creat (str, S_IRWXU);
		/* make BMP header and save it into the file */
		width = BMPPIXELS_NTSC_LRG;	// 256
		height = BMPLINES_NTSC_LRG;	// 192
		if ( SvHeaderBMP(h_File2, width, height, 1)!=0 )
		{
			printf("bmp header write-in failed: \n");
			exit(1);
		}
	  	write (h_File2, status.pbmp, SYS_BMPSIZE);
	  	close (h_File2);
		printf("\timage been saved into %i.jpg and %i.bmp.\n", i, i);
	  }

	  memset(status.pjpg, 0, sizeof(BUFFER));
	  ioctl(bank1, S615FreeBuffer, &status.buffernumber);
	  num=0;

      } else {
	  //printf("\twhen i = %i, num of read-out != sizeof(status)\n", i);
	  i--;
      }
  }

  ioctl(bank1, S615Stop);

  munmap(pb0, sizeof(BUFFER));
  munmap(pb1, sizeof(BUFFER));
  munmap(pb2, sizeof(BUFFER));
  munmap(pb3, sizeof(BUFFER));

  close (bank1);

  return 0;

}

